library Morrowind;

uses kol, data_classes_archive;

const
  FT_ARCHIVE=$0001;
  FT_IMAGESIMPLE=$0002;
  FT_IMAGECOMPLEX=$0004;
  INFO_1='   ...';

var
  FData,FTemp: PStream;

{  }
function MakeTree(Dir: string): Boolean;
begin
  Result := True;
  if Length(Dir) = 0 then Exit;
  if Dir[Length(Dir)]='\' then Delete(Dir,Length(Dir),1);
//  Dir := ExcludeTrailingBackslash(Dir);
  if (Length(Dir) < 3) or DirectoryExists(Dir) or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
  Result := MakeTree(ExtractFilePath(Dir)) and CreateDir(Dir);
end;

//    
procedure GetPluginInfo(var Title,Version,GameName,FilterGroupName,Filters,Extentions,Comments: PChar);
begin
  Title:=PChar('Morrowind Plugin');
  Version:=PChar('v1.01');
  GameName:=PChar('Elder Scroll III: Morrowind');
  FilterGroupName:=PChar('Morrowind (*.bsa)');
  Filters:=PChar('*.bsa');
  Extentions:=PChar('BSA');
  Comments:=PChar('   BSA');
end;

// ID 
function GetPluginID: Integer;
begin
  Result:=$0001;
end;

//   
function GetFileInfo(fName: PChar): Word;
begin
  Result:=FT_ARCHIVE;
end;

//     
function GetFileAutoload(fName: PChar): Boolean;
begin
  Result:=False;
end;

//    
procedure GetArchiveContent(fName: PChar; var fArchive: TArchive; fForm,fLabel,fProgressBar: PControl);
var
  TotalFiles,i,j,fTemp,SavePosition: LongInt;
  c: Char;
  fStr,Extention: string;
  Found: Boolean;
begin
  //  
  fForm.Show;
  fLabel.Caption:=INFO_1+' (1/3)';
  fProgressBar.Progress:=0;
  fForm.Update;
  FData:=NewReadFileStream(fName);
  FData.Seek($08,spBegin);
  FData.Read(TotalFiles,4);
  fArchive.TotalFiles:=TotalFiles;
  fArchive.ArchiveFullFileName:=string(fName);
  for i:=1 to TotalFiles do fArchive.Add('',0,0,0);
  //    
  fLabel.Caption:=INFO_1+' (2/3)';
  fForm.Update;
  FData.Seek($0C,spBegin);
  for i:=1 to TotalFiles do
  begin
    FData.Read(fTemp,4);
    fArchive.Items[i-1].SizeReal:=fTemp;
    fArchive.Items[i-1].SizeCompressed:=fTemp;
    FData.Read(fTemp,4);
    fArchive.Items[i-1].Offset:=fTemp;
    fProgressBar.Progress:=Round(Int(100*(i/TotalFiles)));
  end;
  fLabel.Caption:=INFO_1+' (3/3)';
  fForm.Update;
  //  
  FData.Seek($0C+TotalFiles*8,spBegin);
  SavePosition:=FData.Position;
  for i:=1 to TotalFiles do
  begin
    FData.Seek(SavePosition+(i-1)*4,spBegin);
    FData.Read(fTemp,4);
    FData.Seek(SavePosition+TotalFiles*4+fTemp,spBegin);
    fStr:='';
    repeat
      FData.Read(c,1);
      if not (c=#$00) then fStr:=fStr+c;
    until c=#$00;
    //   
    Extention:=UpperCase(Copy(fStr,DelimiterLast(fStr,'.')+1,3));
    if fArchive.CountFilters=0 then
    begin
      fArchive.AddFilter('*.'+LowerCase(Extention),Extention);
      fArchive.Items[i-1].FilterIndex:=fArchive.CountFilters-1;
    end
    else
    begin
      Found:=False;
      for j:=1 to fArchive.CountFilters do if fArchive.Filters[j-1].Extention=Extention then
      begin
        Found:=True;
        fArchive.Items[i-1].FilterIndex:=j-1;
        Break;
      end;
      if not Found then
      begin
        fArchive.AddFilter('*.'+LowerCase(Extention),Extention);
        fArchive.Items[i-1].FilterIndex:=fArchive.CountFilters-1;
      end;
    end;
    fArchive.SavePosition:=FData.Position+TotalFiles*8;
    //  
    fArchive.Items[i-1].Name:=fStr;
    fArchive.Items[i-1].FlagExtract:=False;
    fProgressBar.Progress:=Round(Int(100*(i/TotalFiles)));
  end;
  fArchive.FlagUseFilters:=True;
  FData.Free;
  fForm.Hide;
end;

procedure UnpackArchive(fName: PChar; fArchive: TArchive; var fForm,fLabel,fProgressBar: PControl);
var
  fBasePath,fFilePath: string;
  i: Integer;
begin
  if fArchive.TotalFiles=0 then Exit;
  //  
  fForm.Show;
  fLabel.Caption:=' ...';
  fProgressBar.Progress:=0;
  fForm.Update;
  fBasePath:=string(fName);
  FData:=NewReadFileStream(fArchive.ArchiveFullFileName);
  if not(fBasePath[Length(fBasePath)]='\') then fBasePath:=fBasePath+'\';
  for i:=1 to fArchive.TotalFiles do
  begin
    if fArchive.Items[i-1].FlagExtract then
    begin
      fFilePath:=fBasePath+ExtractFilePath(fArchive.Items[i-1].Name);
      if not (DirectoryExists(fFilePath)) then MakeTree(fFilePath);
      FTemp:=NewWriteFileStream(fBasePath+fArchive.Items[i-1].Name);
      FData.Seek(fArchive.SavePosition+fArchive.Items[i-1].Offset,spBegin);
      Stream2Stream(FTemp,FData,fArchive.Items[i-1].SizeReal);
      FTemp.Free;
    end;
    fProgressBar.Progress:=Round(Int(100*(i/fArchive.TotalFiles)));
  end;
  FData.Free;
  fForm.Hide;
end;

exports
  GetPluginInfo,
  GetFileInfo,
  GetArchiveContent,
  GetFileAutoload,
  UnpackArchive,
  GetPluginID;

begin

end.
